@@ -1,9 +1,11 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
-from typing import TYPE_CHECKING, Any, Type, cast
+from functools import partial
+from typing import TYPE_CHECKING, Any, Callable, Type, cast
 
 from ._pycrdt import Doc as _Doc
+from ._pycrdt import Subscription
 from ._pycrdt import Transaction as _Transaction
 from ._transaction import ReadTransaction, Transaction
 
@@ -42,6 +44,7 @@ class BaseType(ABC):
     _prelim: Any | None
     _integrated: Any
     _type_name: str
+    _subscriptions: list[Subscription]
 
     def __init__(
         self,
@@ -61,6 +64,7 @@ class BaseType(ABC):
         self._doc = None
         self._prelim = init
         self._integrated = None
+        self._subscriptions = []
 
     @abstractmethod
     def to_py(self) -> Any: ...
@@ -73,9 +77,7 @@ class BaseType(ABC):
 
     def _forbid_read_transaction(self, txn: Transaction):
         if isinstance(txn, ReadTransaction):
-            raise RuntimeError(
-                "Read-only transaction cannot be used to modify document structure"
-            )
+            raise RuntimeError("Read-only transaction cannot be used to modify document structure")
 
     def _integrate(self, doc: Doc, integrated: Any) -> Any:
         prelim = self._prelim
@@ -84,9 +86,7 @@ class BaseType(ABC):
         self._integrated = integrated
         return prelim
 
-    def _do_and_integrate(
-        self, action: str, value: BaseType, txn: _Transaction, *args
-    ) -> None:
+    def _do_and_integrate(self, action: str, value: BaseType, txn: _Transaction, *args) -> None:
         method = getattr(self._integrated, f"{action}_{value.type_name}_prelim")
         integrated = method(txn, *args)
         assert self._doc is not None
@@ -132,6 +132,35 @@ class BaseType(ABC):
     def type_name(self) -> str:
         return self._type_name
 
+    def observe(self, callback: Callable[[Any], None]) -> Subscription:
+        _callback = partial(observe_callback, callback, self.doc)
+        subscription = self.integrated.observe(_callback)
+        self._subscriptions.append(subscription)
+        return subscription
+
+    def observe_deep(self, callback: Callable[[Any], None]) -> Subscription:
+        _callback = partial(observe_deep_callback, callback, self.doc)
+        subscription = self.integrated.observe_deep(_callback)
+        self._subscriptions.append(subscription)
+        return subscription
+
+    def unobserve(self, subscription: Subscription) -> None:
+        self._subscriptions.remove(subscription)
+        subscription.drop()
+
+
+def observe_callback(callback: Callable[[Any], None], doc: Doc, event: Any):
+    _event = event_types[type(event)](event, doc)
+    with doc._read_transaction(event.transaction):
+        callback(_event)
+
+
+def observe_deep_callback(callback: Callable[[Any], None], doc: Doc, events: list[Any]):
+    for idx, event in enumerate(events):
+        events[idx] = event_types[type(event)](event, doc)
+    with doc._read_transaction(event.transaction):
+        callback(events)
+
 
 class BaseEvent:
     __slots__ = ()
