@@ -97,9 +97,7 @@ class Doc(BaseDoc):
                 key: (
                     None
                     if val is None
-                    else cast(Type[BaseType], base_types[type(val)])(
-                        _integrated=val, _doc=self
-                    )
+                    else cast(Type[BaseType], base_types[type(val)])(_integrated=val, _doc=self)
                 )
                 for key, val in self._doc.roots(txn._txn).items()
             }
